@@ -251,8 +251,15 @@ class _ExtractPerformer(object):
         finder = similarfinder.SimilarFinder(self.info.pymodule)
         matches = []
         for start, end in regions:
-            matches.extend((finder.get_matches(collector.body_pattern,
-                                               collector.checks, start, end)))
+            region_matches = finder.get_matches(collector.body_pattern,
+                                               collector.checks, start, end)
+            # Don't extract overlapping regions
+            last_match_end = -1
+            for region_match in region_matches:
+                start, end = region_match.get_region()
+                if last_match_end < start:
+                    matches.append(region_match)
+                    last_match_end = end
         collector.matches = matches
 
     def _where_to_search(self):
@@ -540,7 +547,7 @@ class _ExtractMethodParts(object):
         # if not make_global, do not pass any global names; they are
         # all visible.
         if self.info.global_ and not self.info.make_global:
-            return ()
+            return list(self.info_collector.read & self.info_collector.postread & self.info_collector.written)
         if not self.info.one_line:
             result = (self.info_collector.prewritten &
                       self.info_collector.read)
@@ -655,6 +662,11 @@ class _FunctionInformationCollector(object):
         for child in node.targets:
             ast.walk(child, self)
 
+    def _AugAssign(self, node):
+        ast.walk(node.value, self)
+        self._read_variable(node.target.id, node.target.lineno)
+        self._written_variable(node.target.id, node.target.lineno)
+
     def _ClassDef(self, node):
         self._written_variable(node.name, node.lineno)
 
