@@ -1,10 +1,12 @@
 import collections
+import numbers
 import re
 import warnings
 
 from rope.base import ast, codeanalyze, exceptions
 from rope.base.utils import pycompat
 
+
 try:
     basestring
 except NameError:
@@ -123,7 +125,10 @@ class _PatchingASTWalker(object):
                     # semicolon in except
                     region = self.source.consume_except_as_or_semicolon()
                 else:
-                    region = self.source.consume(child)
+                    if hasattr(ast, 'JoinedStr') and isinstance(node, (ast.JoinedStr, ast.FormattedValue)):
+                        region = self.source.consume_joined_string(child)
+                    else:
+                        region = self.source.consume(child)
                 child = self.source[region[0]:region[1]]
                 token_start = region[0]
             if not first_token:
@@ -253,6 +258,13 @@ class _PatchingASTWalker(object):
         children.extend(['=', node.value])
         self._handle(node, children)
 
+    def _AnnAssign(self, node):
+        children = [node.target, ':', node.annotation]
+        if node.value is not None:
+            children.append('=')
+            children.append(node.value)
+        self._handle(node, children)
+
     def _Repr(self, node):
         self._handle(node, ['`', node.value, '`'])
 
@@ -268,15 +280,38 @@ class _PatchingASTWalker(object):
         self._handle(node, ['break'])
 
     def _Call(self, node):
+        def _arg_sort_key(node):
+          if isinstance(node, ast.keyword):
+            return (node.value.lineno, node.value.col_offset)
+          return (node.lineno, node.col_offset)
+
         children = [node.func, '(']
-        args = list(node.args) + node.keywords
-        children.extend(self._child_nodes(args, ','))
+        unstarred_args = []
+        starred_and_keywords = list(node.keywords)
+        for i, arg in enumerate(node.args):
+          if hasattr(ast, 'Starred') and isinstance(arg, ast.Starred):
+            starred_and_keywords.append(arg)
+          else:
+            unstarred_args.append(arg)
         if getattr(node, 'starargs', None):
-            if args:
-                children.append(',')
-            children.extend(['*', node.starargs])
+          starred_and_keywords.append(node.starargs)
+        starred_and_keywords.sort(key=_arg_sort_key)
+        children.extend(self._child_nodes(unstarred_args, ','))
+
+        # positional args come before keywords, *args comes after all
+        # positional args, and **kwargs comes last
+        if starred_and_keywords:
+          if len(children) > 2:
+            children.append(',')
+          for i, arg in enumerate(starred_and_keywords):
+            if arg == getattr(node, 'starargs', None):
+              children.append('*')
+            children.append(arg)
+            if i + 1 < len(starred_and_keywords):
+              children.append(',')
+
         if getattr(node, 'kwargs', None):
-            if args or node.starargs is not None:
+            if len(children) > 2:
                 children.append(',')
             children.extend(['**', node.kwargs])
         children.append(')')
@@ -308,12 +343,74 @@ class _PatchingASTWalker(object):
     def _Delete(self, node):
         self._handle(node, ['del'] + self._child_nodes(node.targets, ','))
 
+    def _Constant(self, node):
+        if isinstance(node.value, basestring):
+            self._handle(node, [self.String])
+            return
+
+        if any(node.value is v for v in [True, False, None]):
+            self._handle(node, [str(node.value)])
+            return
+
+        if isinstance(node.value, numbers.Number):
+            self._handle(node, [self.Number])
+            return
+
+        if node.value is Ellipsis:
+            self._handle(node, ['...'])
+            return
+
+        assert False
+
     def _Num(self, node):
         self._handle(node, [self.Number])
 
     def _Str(self, node):
         self._handle(node, [self.String])
 
+    def _Bytes(self, node):
+        self._handle(node, [self.String])
+
+    def _JoinedStr(self, node):
+        def start_quote_char():
+            possible_quotes = [(self.source.source.find(q, start, end), q) for q in QUOTE_CHARS]
+            quote_pos, quote_char = min((pos, q) for pos, q in possible_quotes if pos != -1)
+            return self.source[start:quote_pos + len(quote_char)]
+
+        def end_quote_char():
+            possible_quotes = [(self.source.source.rfind(q, start, end), q) for q in reversed(QUOTE_CHARS)]
+            _, quote_pos, quote_char = max((len(q), pos, q) for pos, q in possible_quotes if pos != -1)
+            return self.source[end - len(quote_char):end]
+
+        QUOTE_CHARS = ['"""', "'''", '"', "'"]
+        offset = self.source.offset
+        start, end = self.source.consume_string(
+            end=self._find_next_statement_start(),
+        )
+        self.source.offset = offset
+
+        children = []
+        children.append(start_quote_char())
+        for part in node.values:
+            if isinstance(part, ast.FormattedValue):
+                children.append(part)
+        children.append(end_quote_char())
+        self._handle(node, children)
+
+    def _FormattedValue(self, node):
+        children = []
+        children.append('{')
+        children.append(node.value)
+        if node.format_spec:
+            children.append(':')
+            for val in node.format_spec.values:
+                if isinstance(val, ast.FormattedValue):
+                    children.append(val.value)
+                else:
+                    children.append(val.s)
+        children.append('}')
+        self._handle(node, children)
+
     def _Continue(self, node):
         self._handle(node, ['continue'])
 
@@ -322,7 +419,11 @@ class _PatchingASTWalker(object):
         children.append('{')
         if node.keys:
             for index, (key, value) in enumerate(zip(node.keys, node.values)):
-                children.extend([key, ':', value])
+                if key is None:
+                    # PEP-448 dict unpacking: {a: b, **unpack}
+                    children.extend(['**', value])
+                else:
+                    children.extend([key, ':', value])
                 if index < len(node.keys) - 1:
                     children.append(',')
         children.append('}')
@@ -334,6 +435,10 @@ class _PatchingASTWalker(object):
     def _Expr(self, node):
         self._handle(node, [node.value])
 
+    def _NamedExpr(self, node):
+        children = [node.target, ':=', node.value]
+        self._handle(node, children)
+
     def _Exec(self, node):
         children = []
         children.extend(['exec', node.body])
@@ -717,11 +822,11 @@ class _Source(object):
         self.source = source
         self.offset = 0
 
-    def consume(self, token):
+    def consume(self, token, skip_comment=True):
         try:
             while True:
                 new_offset = self.source.index(token, self.offset)
-                if self._good_token(token, new_offset):
+                if self._good_token(token, new_offset) or not skip_comment:
                     break
                 else:
                     self._skip_comment()
@@ -732,9 +837,16 @@ class _Source(object):
         self.offset = new_offset + len(token)
         return (new_offset, self.offset)
 
+    def consume_joined_string(self, token):
+        new_offset = self.source.index(token, self.offset)
+        self.offset = new_offset + len(token)
+        return (new_offset, self.offset)
+
     def consume_string(self, end=None):
         if _Source._string_pattern is None:
-            original = codeanalyze.get_string_pattern()
+            string_pattern = codeanalyze.get_string_pattern()
+            formatted_string_pattern = codeanalyze.get_formatted_string_pattern()
+            original = r'(?:%s)|(?:%s)' % (string_pattern, formatted_string_pattern)
             pattern = r'(%s)((\s|\\\n|#[^\n]*\n)*(%s))*' % \
                       (original, original)
             _Source._string_pattern = re.compile(pattern)
